/***************************************************************************
                          baseplugin.h  -  description                              
                             -------------------                                         
    begin                : Tue Oct 5 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef BASEPLUGIN_H
#define BASEPLUGIN_H

#include <qobject.h>
#include <qstring.h>
#include <qlist.h>
#include <basefileinfo.h>
/**
  *@author Henrik Stormer
  */

class ST
{
 public:
 BaseFileInfo* Info;
 QString     Source;
 QString     Target;
};


class QWidget;
class MyListView;
class ProgressDialog;
class QPopupMenu;
class KToolBar;
class KAccel;
class KNCApp;
class BasePluginFactory;

class BasePlugin
{
public:
  enum FileOperation {FileCopy=1, FileDelete=2, FileRename=4, ChangeAttribute=8, NewDirectory=16};

  BasePlugin();
  virtual ~BasePlugin();
  virtual BasePluginFactory* getPluginFactory() = 0;
  //these methods should be overloaded to signal that a Plugin is responsible
  //for this path
  virtual bool isResponsible(QString Path, QString SubPath) = 0;

  virtual bool create(QString Path, MyListView* ListBox, QString CurrentItemName, bool ShowHiddenFiles) {}
  virtual QString cdUp(QString Path, QString& Name) {}
  virtual QString cdDown(QString Path, QString DownPath) {}

  virtual bool sourceBeforeFileOperation(QString Path, int Operation) {return true;}
  virtual bool targetBeforeFileOperation(QString Path, int Operation) {return true;}
  virtual void sourceAfterFileOperation(QString Path, int Operation)  {}
  virtual void targetAfterFileOperation(QString Path, int Operation)  {}

  virtual bool copyIn(ProgressDialog* PDialog, ST st, bool& bOverwrite, bool& bStop) {}
  virtual bool makeDirectory(QString Path, QString Dir) {};

  virtual void recurseSubdir(QString Dir, QList<BaseFileInfo>& FileList, double& countFiles, double& sizeAll) {}

  virtual void insertEntry(MyListView* ListBox, QString Dir, QString Name) {}
  virtual void removeEntry(MyListView* ListBox, QString Dir, QString Name) {}

  virtual bool getGroups(QStrList& GroupList) {return false;}
  virtual bool getOwners(QStrList& OwnerList) {return false;}

  virtual QString getType() {return "BasePlugin";}
  virtual BasePlugin* clone(QString Dir, QString SubDir) = 0;

  virtual void getsActive(MyListView* ListBox) {}
  virtual void getsInactive(MyListView* ListBox) {delete this;}
};



#endif















