/***************************************************************************
                          ftppluginfactory.cpp  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Henrik Stormer
    email                : stormer@ifi.unizh.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ftppluginfactory.h"
#define PRIORITY 5
#define ID_FTP_CONNECTION 5200
#define ID_FTP_DISCONNECT 5201
#include <kaccel.h>
#include <kapp.h>
#include "ftpconnectiondialog.h"
#include "pluginadmin.h"
#include "listboxdialog.h"
#include <iostream.h>

const int StdColumnSize = 9;
const int StdColumn [StdColumnSize] = {2,0,4,5,3,6,7,8,1};

FtpConfiguration::FtpConfiguration()
{
  _Anonymous = true;
  _Port = 21;
  _ColumnOrder.setAutoDelete(true);
  for (int i=0;i<StdColumnSize;i++) _ColumnOrder.append(new short(StdColumn[i]));
}
FtpConfiguration::FtpConfiguration(QString Name) : _Name(Name)
{
  _Anonymous = true;
  _Port = 21;
  _ColumnOrder.setAutoDelete(true);
  for (int i=0;i<StdColumnSize;i++) _ColumnOrder.append(new short(StdColumn[i]));
}

FtpConfiguration::FtpConfiguration(KConfig* Config, QString Name)
{
 _ColumnOrder.setAutoDelete(true);
 readConfig(Config, Name);
}
bool FtpConfiguration::readConfig(KConfig* Config, QString Name)
{
 _Name = Name;
 Config->setGroup("Ftp Configuration: " + Name);
 _Host = Config->readEntry("Host","");
 _Dir = Config->readEntry("Dir", "/");
 _Port = Config->readNumEntry("Port", 21);
 _Anonymous = Config->readBoolEntry("Anonymous", true);
 if (!_Anonymous)
 {
   _User = Config->readEntry("User");
   _Password = Config->readEntry("Password");
 }
 QStrList ColumnStrList;
 Config->readListEntry("Column Order", ColumnStrList);
 QListIterator<char> it(ColumnStrList);
 while (it.current())
 {
   QString T = (it.current());
   short TS = T.toShort();
   if ((0 <= TS) && (ColumnSize > TS))
     _ColumnOrder.append(new short(TS));
   ++it;
 }
 return true;
}
bool FtpConfiguration::writeConfig(KConfig* Config)
{
 Config->setGroup("Ftp Configuration: " + _Name);
 Config->writeEntry("Host", _Host);
 Config->writeEntry("Dir", _Dir);
 Config->writeEntry("Port", _Port);
 Config->writeEntry("Anonymous", _Anonymous);
 if (!_Anonymous)
 {
   Config->writeEntry("User", _User);
   Config->writeEntry("Password", _Password);
 }
 QStrList ColumnStrList;
 QListIterator<short> it(_ColumnOrder);
 while (it.current())
 {
   ColumnStrList.append(QString().setNum(*(it.current())));
   ++it;
 }

 Config->writeEntry("Column Order", ColumnStrList);
 return true;
}


FtpPluginFactory::FtpPluginFactory(QMultiLineEdit* ProtocolView, KNCApp* App)
   : BasePluginFactory(PRIORITY, App), _ProtocolView(ProtocolView)
{
  KConfig* Config=kapp->getConfig();
  QStrList ConnectionStrList;
  _ConnectionList.setAutoDelete(true);
  Config->setGroup("Ftp Configuration");
  int num = Config->readListEntry("ConnectionList", ConnectionStrList);
  for (int i=0;i<num;i++)
  {
    FtpConfiguration* FtpConfig = new FtpConfiguration(Config, ConnectionStrList.at(i));
    _ConnectionList.append(FtpConfig);
  }
}


FtpPluginFactory::~FtpPluginFactory()
{
 KConfig* Config=kapp->getConfig();
 QStrList ConnectionStrList;
 QListIterator<FtpConfiguration> it(_ConnectionList);
 for (;it.current();++it)
 {
   ConnectionStrList.append(it.current()->_Name);
   it.current()->writeConfig(Config);
 }
 Config->setGroup("Ftp Configuration");
 Config->writeEntry("ConnectionList", ConnectionStrList);
//  int num = Config->readListEntry("ConnectionList", ConnectionList);

}


bool FtpPluginFactory::isResponsible(QString Path, QString SubPath)
{
 if (Path.find("#ftp:") != -1) return true;

 if (SubPath != "")
 {
   if (SubPath.right(4) == ".ftp") return true;
 }
 if (Path.left(6) == "ftp://") return true;
 return false;
}


bool FtpPluginFactory::addMenuItems(QPopupMenu* menu)
{
 menu->insertItem(i18n("Ftp &Connection"), ID_FTP_CONNECTION);
 menu->insertItem(i18n("Ftp &Disconnect"), ID_FTP_DISCONNECT);
 return true;
}

bool FtpPluginFactory::addToolbarItems(KToolBar*)
{
 return false;
}

bool FtpPluginFactory::commandCallback(int id)
{
 switch (id)
 {
  case ID_FTP_CONNECTION: slotFtpConnection(); return true;
  case ID_FTP_DISCONNECT: slotFtpDisconnect(); return true;
  default: return false;
 }
}

bool FtpPluginFactory::statusCallback(int id, QString& StatusMsg)
{
 switch (id)
 {
  case ID_FTP_CONNECTION: StatusMsg = i18n("Shows a dialog to establish a ftp connection"); return true;
  case ID_FTP_DISCONNECT: StatusMsg = i18n("Shows a dialog to disconnect a ftp connection"); return true;
  default: return false;
 }
}

void FtpPluginFactory::initKeyAccel (KAccel* accel, KNCApp* App)
{
  accel->insertItem (i18n("Ftp Connection"), "Ftp Connection", CTRL+Key_C);
  accel->insertItem (i18n("Ftp Disconnect"), "Ftp Disconnect", CTRL+Key_D);
}

void FtpPluginFactory::changeMenuAccel (KAccel* accel, QPopupMenu* menu)
{
  accel->disconnectItem ("Ftp Connection", this, SLOT (slotFtpConnection() ) );
  accel->connectItem ("Ftp Connection", this, SLOT (slotFtpConnection() ) );

  accel->disconnectItem ("Ftp Disconnect", this, SLOT (slotFtpDisconnect() ) );
  accel->connectItem ("Ftp Disconnect", this, SLOT (slotFtpDisconnect() ) );

  accel->changeMenuAccel(menu, ID_FTP_CONNECTION ,"Ftp Connection" );
  accel->changeMenuAccel(menu, ID_FTP_DISCONNECT ,"Ftp Disconnect" );
}

void FtpPluginFactory::slotFtpConnection()
{
  FtpConnectionDialog Dialog(_ConnectionList);
  if (Dialog.exec())
  {
    FtpConfiguration* Config = Dialog.getConfiguration();
    MyListView* ActListBox = getApp()->getActListBox();

//    FtpPlugin* Plugin = new FtpPlugin(this, _ProtocolView, Config->_Host, Config->_Dir,
//                                   Config->_User, Config->_Password, Config->_Port, ActListBox->getDir());

//    PluginAdmin::instance().addPluginInstance(Plugin);
    ActListBox->setDirectory("ftp://" + Config->_Host + Config->_Dir);
  }
}

BasePlugin* FtpPluginFactory::generatePlugin(QString Dir, QString SubDir)
{
 FtpPlugin* Plugin = new FtpPlugin(this, _ProtocolView);
 registerPlugin(Plugin);
 return Plugin;
}

FtpConfiguration* FtpPluginFactory::getFtpConfiguration(QString Host)
{
 QListIterator<FtpConfiguration> it(_ConnectionList);
 for (;it.current();++it)
   if (it.current()->_Host == Host) return it.current();

 return NULL;
}

void FtpPluginFactory::slotFtpDisconnect()
{
  QStrList Connections;
  QListIterator<FtpPlugin> it(_PluginList);
  for (;it.current();++it)
    Connections.append(it.current()->getHost());
  ListBoxDialog Dialog(Connections, i18n("Disconnect"));
  if (Dialog.exec())
  {
    FtpPlugin* Plugin = _PluginList.take(Dialog.getType());
    if (Plugin)
    {
       Plugin->_ActListBox->setDirectory(Plugin->_OldDir);
    }
  }
}