/***************************************************************************
                          myfileinfo.h  -  description                              
                             -------------------                                         
    begin                : Sat Sep 25 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef MYFILEINFO_H
#define MYFILEINFO_H

#include <qfileinfo.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <basefileinfo.h>
#include "ktar.h"

/**
  *@author Henrik Stormer
  *
  * This class wipes out some unpleasant QFileInfo behaviours
  */


class GeneralFileInfo : public BaseFileInfo
{
public: 
	GeneralFileInfo();
  GeneralFileInfo(QString Path, QString Name, struct stat& st);
	virtual ~GeneralFileInfo();

  virtual QString getAttrib() const;
  virtual void setOwner(QString Owner);
  virtual void setGroup(QString Group);

  virtual QString infoAsString ();
  virtual QString type() {return "GeneralFile";}
  virtual bool    exists() const {return _bExists;}
  virtual GeneralFileInfo* clone ()  {return new GeneralFileInfo(*this);}

  virtual void setAttribute(long Attribute);
  virtual long getAttribute() const;

  virtual bool accessRead() const {return (access(filePath(), R_OK)==0);}
  virtual bool accessWrite() const  {return (access(filePath(), W_OK)==0);}
  virtual bool accessExecute() const {return (access(filePath(), X_OK)==0);}

  virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
  virtual bool rename(ProgressDialog* PFile, QString Target, bool& bOverwrite, bool& stop);
  virtual bool remove(DeleteDialog* PFile, bool& stop);
  virtual QPixmap getIcon();

protected:
  mode_t  _Mode;
  bool   _bExists;
  uid_t  _UserId;
  gid_t  _GroupId;
};

class GeneralDirInfo : public GeneralFileInfo
{
 public:
	GeneralDirInfo() : GeneralFileInfo(), _bSizeSet(false) {}
  GeneralDirInfo(QString Path, QString Name, struct stat& st) : GeneralFileInfo(Path, Name, st), _bSizeSet(false) {}
	virtual ~GeneralDirInfo() {}
  virtual QString getSize() const;
  virtual QString infoAsString () {return GeneralFileInfo::infoAsString();}
  virtual QString type() {return "GeneralDir";}
  virtual GeneralDirInfo* clone ()  {return new GeneralDirInfo(*this);}

  virtual bool isFile() const {return false;}
  virtual bool isDir() const {return true;}

  virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
  virtual QPixmap getIcon();
 protected:
  bool _bSizeSet;
};


class GeneralLinkInfo : public GeneralFileInfo
{
 public:
	GeneralLinkInfo() : GeneralFileInfo() {}
  GeneralLinkInfo(QString Path, QString Name, struct stat& st);
	virtual ~GeneralLinkInfo() {}

  virtual QString infoAsString ();
  virtual QString type() {return "GeneralLink";}
  virtual GeneralLinkInfo* clone ()  {return new GeneralLinkInfo(*this);}

  virtual bool isFile() const {return _bFileLink;}
  virtual bool isDir() const {return !_bFileLink;}
  virtual bool isLink() const {return true;}

  QString getLink() {return _Link;}
  virtual QPixmap getIcon();

  virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
 protected:
  QString _Link;
  bool    _bFileLink;
};

class GeneralDeviceInfo : public GeneralFileInfo
{
 public:
	GeneralDeviceInfo() : GeneralFileInfo() {}
  GeneralDeviceInfo(QString Path, QString Name, struct stat& st) : GeneralFileInfo(Path, Name, st) {}
	virtual ~GeneralDeviceInfo() {}

//  virtual QString infoAsString ();
  virtual QString type() {return "GeneralDevice";}
  virtual GeneralDeviceInfo* clone ()  {return new GeneralDeviceInfo(*this);}


  virtual QPixmap getIcon();
  virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
  virtual bool rename(ProgressDialog* PFile, QString Target, bool& stop);
  virtual bool remove(DeleteDialog* PFile, bool& stop);
};

class GeneralOtherInfo : public GeneralFileInfo
{
 public:
	GeneralOtherInfo() : GeneralFileInfo() {}
  GeneralOtherInfo(QString Path, QString Name, struct stat& st) : GeneralFileInfo(Path, Name, st) {}
	virtual ~GeneralOtherInfo() {}

//  virtual QString infoAsString ();
  virtual QString type() {return "GeneralOther";}
  virtual GeneralOtherInfo* clone ()  {return new GeneralOtherInfo(*this);}

  virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
  virtual bool rename(ProgressDialog* PFile, QString Target, bool& stop);
  virtual bool remove(DeleteDialog* PFile, bool& stop);
};

/*
class TarEntry : public MyFileInfo
{
 public:
  TarEntry(KTarEntry* TarEntry, QString Path)
     : MyFileInfo(), _TarEntry(TarEntry)
  {
     _Name = _TarEntry->name();
     _Path = Path;
  }

  virtual QString fileName() const {return _Name;}
  virtual QString filePath() const {return _Path + _Name;}
  virtual QString dirPath() const {return _Path;}
  virtual QString attrib() const;
  virtual QDateTime lastModified() const {return _LastModified;}
  virtual QDateTime lastRead() const {return _LastRead;}
 protected:
   KTarEntry* _TarEntry;
};


class TarFileInfo : public TarEntry
{
  public:
   TarFileInfo (KTarFile* TarFile, QString Path)
      : TarEntry(TarFile, Path), _TarFile(TarFile)
   {
     _Size = _TarFile->size();
   }
   virtual ~TarFileInfo() {}


   virtual bool isDir() {return true;}
   virtual bool isFile() {return false;}
  protected:
   KTarFile* _TarFile;
};

class TarDirInfo : public TarEntry
{
  public:
   TarDirInfo(KTarDirectory* TarDir, QString Path)
    : TarEntry(TarDir, Path), _TarDir(TarDir)
  { }

   virtual  QString getSizeAsString() {return "<DIR>";}
   virtual ~TarDirInfo() {}

   virtual bool isDir() {return true;}
   virtual bool isFile() {return false;}
  protected:
   KTarDirectory* _TarDir;
};
*/
#endif





























































