/***************************************************************************
                          generalfileplugin.cpp  -  description                              
                             -------------------                                         
    begin                : Tue Oct 5 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "generalfileplugin.h"
#include "qdir.h"
#include "qmessagebox.h"
#include <sys/types.h>
#include <dirent.h>

#include "qstring.h"
#include "mylistview.h"
#include "knc.h"
#include "qapplication.h"
#include "helpfunc.h"

#include <grp.h>
#include <pwd.h>

#define PRIORITY 1

GeneralFilePlugin::GeneralFilePlugin(GeneralFilePluginFactory* Factory)
 : BasePlugin(), _Factory(Factory)
{
}
GeneralFilePlugin::~GeneralFilePlugin()
{
}

GeneralFilePluginFactory::GeneralFilePluginFactory()
 : BasePluginFactory(PRIORITY)
{
}

GeneralFilePluginFactory::~GeneralFilePluginFactory()
{
}


bool GeneralFilePluginFactory::isResponsible(QString Path, QString SubPath)
{
 if (SubPath != "") Path += SubPath;
 QDir Dir(Path);
 if (Dir.exists()) return true;
 return false;
}

bool GeneralFilePlugin::isResponsible(QString Path, QString SubPath)
{
 if (SubPath != "") Path += SubPath;
 QDir Dir(Path);
 if (Dir.exists()) return true;
 return false;
}

BasePluginFactory* GeneralFilePlugin::getPluginFactory()
{
 return _Factory;
}

bool GeneralFilePlugin::create(QString Dir, MyListView* ListBox, QString CurrentItemName, bool ShowHiddenFiles)
{
  ListBox->clear();

  QDir TDir (Dir);
  if (!TDir.exists()) return false;

  struct dirent *d;
  MyListViewItem* Item;
  BaseFileInfo* f;
  DIR* dp = opendir(Dir);

  //this happens if someone sets execute but no read access for a directory
  //this is really stupid, but a crash has to be avoided
  if (dp == NULL)
  {
     f = generateFileInfo(Dir, "..");
     Item = ListBox->insertItem(f);
     return true;
  }

  while ((d = readdir(dp)) != NULL)
  {
    QString Name = d->d_name;
    if (Name != ".")
    {
      if ((Name == "..") || (ShowHiddenFiles) || (Name.left(1) != "."))
      {
         f = generateFileInfo(Dir, Name);
         Item = ListBox->insertItem(f);
         if (Name == CurrentItemName)
               ListBox->setCurrentItem((QListViewItem*)Item);
      }
    }
  }
  closedir(dp);
  return true;
}

BaseFileInfo* GeneralFilePlugin :: generateFileInfo(QString Dir, QString Name)
{
  struct stat st;
  if (lstat (Dir + Name, &st) < 0) return NULL;
  BaseFileInfo* f;

  if (S_ISREG(st.st_mode))  f = new GeneralFileInfo(Dir, Name, st);
  if (S_ISDIR(st.st_mode))  f = new GeneralDirInfo(Dir, Name, st);
  if (S_ISLNK(st.st_mode))  f = new GeneralLinkInfo(Dir, Name, st);
  if (S_ISCHR(st.st_mode))  f = new GeneralDeviceInfo(Dir, Name, st);
  if (S_ISBLK(st.st_mode))  f = new GeneralDeviceInfo(Dir, Name, st);
  if (S_ISSOCK(st.st_mode)) f = new GeneralOtherInfo(Dir, Name, st);
  if (S_ISFIFO(st.st_mode)) f = new GeneralOtherInfo(Dir, Name, st);
  return f;
}

QString GeneralFilePlugin::cdUp(QString Path, QString& DirName)
{
  cutUpPath(Path, Path, DirName);
  return Path;
}

QString GeneralFilePlugin::cdDown(QString Path, QString DownPath)
{
  return Path + DownPath + "/";
}

void GeneralFilePlugin::recurseSubdir(QString Dir, QList<BaseFileInfo>& FileList, double& countFiles, double& sizeAll)
{
  struct dirent *d;
  struct stat st;

  if ((access(Dir, R_OK)==0) && (access(Dir, X_OK)==0))
  {
    qApp->processEvents();
    DIR* dp = opendir(Dir);
    while ((d = readdir(dp)) != NULL)
    {
      QString Name = d->d_name;
      if ((Name == ".") || (Name == "..")) continue;

      lstat (Dir + d->d_name, &st);
      BaseFileInfo* f = generateFileInfo(Dir, d->d_name);
      FileList.append(f);

      if ((f->isDir()) && (!f->isLink()))
        recurseSubdir(Dir + d->d_name + "/", FileList, countFiles, sizeAll);
      else
      {
        countFiles++;
        sizeAll += st.st_size;
      }
    }
    closedir(dp);
  }
}

void GeneralFilePlugin :: insertEntry(MyListView* ListBox, QString Dir, QString Name)
{
  bool Selected = false;
  MyListViewItem* Item;
  if (ListBox->getDir() != Dir) return;
  BaseFileInfo* f = generateFileInfo(Dir, Name);
  if (f)
  {
    if ((Item = ListBox->findItem(Name))!=NULL)
    {
      Selected = Item->isSelected();
      ListBox->removeItem(Item);
    }
    Item = ListBox->insertItem(f);
    ListBox->setSelected(Item, Selected);
  }
}

void GeneralFilePlugin :: removeEntry(MyListView* ListBox, QString Dir, QString Name)
{
  if (ListBox->getDir() != Dir) return;
  MyListViewItem* Item = ListBox->findItem(Name);
  ListBox->removeItem(Item);
}


bool GeneralFilePlugin :: makeDirectory(QString Path, QString NewDir)
{
 QDir Dir(Path);
 if (!Dir.mkdir (NewDir, false))
 {
   QString Message;
   QMessageBox::critical( NULL, i18n("KNC Problem"), Message.sprintf(i18n("Unable to create Directory %s."), NewDir.data()));
   return false;
 }
 return true;
}


bool GeneralFilePlugin::getOwners(QStrList& OwnerList)
{
 QString Owner;
 __uid_t OwnerId = geteuid();
 struct passwd* pw = getpwuid(OwnerId);
 Owner = pw->pw_name;

 if (Owner == "root")
 {
   setpwent();
   struct passwd* pw;
   while ((pw = getpwent()) != NULL)
      OwnerList.append(pw->pw_name);
   endpwent();
   return true;
 }
 else
 {
   OwnerList.append(Owner);
   return false;
 }
}

bool GeneralFilePlugin::getGroups(QStrList& GroupList)
{
 QString Owner;
 __uid_t OwnerId = geteuid();
 struct passwd* pw = getpwuid(OwnerId);
 Owner = pw->pw_name;

 setgrent();
 struct group* ge;
 while ((ge = getgrent()) != NULL)
 {
   if (Owner == "root") GroupList.append(ge->gr_name);
   else
   {
     char ** members = ge->gr_mem;
     char * member;
	   while ((member = *members) != 0L)
     {
       if (Owner == member) GroupList.append(ge->gr_name);
       ++members;
     }
   }
 }
 endgrent();
 return true;
}





