/***************************************************************************
                          knc.h  -  description                              
                             -------------------                                         
    begin                : Sat Jul 31 18:15:05 MEST 1999
                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef KNC_H
#define KNC_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


// include files for KDE 
#include <kapp.h>
#include <ktmainwindow.h>
#include <kaccel.h>

//needed QT Files
#include <qdir.h>
#include <qlistview.h>

#include "mylistview.h"
#include "generalfileinfo.h"

// forward declaration of the KNC classes
class KNCDoc;
class KNewPanner;
class QMultiLineEdit;
class QLineEdit;
class KSimpleConfig;
class QFileInfo;
class MyCommandWidget;

class MyListViewItem : public QListViewItem
{
  public:
   MyListViewItem ( MyListView * parent, BaseFileInfo* Info )
   : QListViewItem ((QListView*)parent), _MyListView(parent), _FileInfo(Info) {setPixmap(0,Info->getIcon());}

   MyListViewItem ( MyListView * parent, BaseFileInfo* Info, QListViewItem * after )
   : QListViewItem ((QListView*)parent, after), _MyListView(parent), _FileInfo(Info)  {setPixmap(0,Info->getIcon());}

   MyListViewItem ( MyListView * parent, BaseFileInfo* Info, const char * c1, const char * c2 = 0, const char * c3 = 0, const char * c4 = 0, const char * c5 = 0, const char * c6 = 0, const char * c7 = 0, const char * c8 = 0 )
   : QListViewItem ((QListView*)parent, c1, c2, c3, c4, c5, c6, c7, c8 ), _MyListView(parent), _FileInfo(Info) {setPixmap(0,Info->getIcon());}

   BaseFileInfo* getFileInfo() {return _FileInfo;}

   virtual ~MyListViewItem () {delete (_FileInfo);}
   virtual void paintFocus( QPainter * p, const QColorGroup & cg, const QRect & r );
   virtual const char* key(int column, bool ascending) const;
  protected:
  private:
   MyListView* _MyListView;
   BaseFileInfo* _FileInfo;
};


/**
  * The base class for KNC application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar.
  * KNCApp reimplements the methods that KTMainWindow provides for main window handling and supports
  * full session management as well as keyboard accelerator configuration by using KAccel.
	* @see KTMainWindow
	* @see KApplication
	* @see KConfig
	* @see KAccel
	*
	* @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
	* @version KDevelop version 0.4 code generation
  */
const int MAX_BOOKMARKS = 64;

class KNCApp : public KTMainWindow
{
  Q_OBJECT

public:
  /** construtor of KNCApp, calls all init functions to create the application.
    * @see initMenuBar initToolBar
    */
  KNCApp();
  /**  */
  void        setActListBox (bool LeftBox);
  MyListView* getActListBox () {return _ActListBox;}
  /**  */
  void makeItemVisible(MyListViewItem* Item);

//  MyListViewItem* findItem(QString Name);

  void RightButtonClicked(QListViewItem* Item);

  ~KNCApp();
  /** enables menuentries/toolbar items
  	*/
  void enableCommand(int id_);
  /** disables menuentries/toolbar items
  	*/
  void disableCommand(int id_);
	/** add a opened file to the recent file list and update recent_file_menu
		*/
  void select(bool Selection, QString Text);

  void copyMove(bool bMove);
  //tempfiles

protected:
  void initPlugins ();
  void createSourceTargetList(QList<ST>& SourceTargetList, QList<BaseFileInfo>& FileList, QString SourcePath, QString TargetPath, QString Pattern);
 	/** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
		* file
		*/ 	
	void saveOptions();
	/** read general Options again and initialize all variables like the recent file list
	  */
	void readOptions();
	/** initKeyAccel creates the keyboard accelerator items for the available slots and changes the menu accelerators.
		* @see KAccel
		*/
	void initKeyAccel();
  void changeMenuAccels();
  void initMenuBar();
  void initToolBar();
  void initStatusBar();
  void initView();

	virtual bool queryExit();
  void exec(QString command);
  void refresh (bool left, QString CurrentItemName="");

  void insertItem(QString Path, QString File);
  void removeItem(QString Path, QString File);
 public slots:
  /** switch argument for slot selection by menu or toolbar ID */
  void commandCallback(int id_);
  /** switch argument for Statusbar help entries on slot selection. Add your ID's help here for toolbars and menubar entries. */
  void statusCallback(int id_);
  /** open a new application window by creating a new instance of KNCApp */

  void slotLeftHidden();
  void slotLeftSortName();
  void slotLeftSortSize();
  void slotLeftSortAttrib();
  void slotLeftSortOwner();
  void slotLeftSortGroup();
  void slotLeftSortLastModTime();
  void slotLeftSortLastReadTime();
  void slotLeftRefresh();

  void slotCommandView();
  void slotCommandEdit();
  void slotCommandCopy();
  void slotCommandMove();
  void slotCommandNewDir();
  void slotCommandDelete();
  void slotCommandSelect();
  void slotCommandUnselect();
  void slotCommandOpenTerminal();
  void slotCommandOpenFindUtility();
  void slotCommandOpenUserMenu();
  void slotCommandExecuteWith();
  void slotCommandChangeAttributes();
  void slotCommandInvokeCommandLine();
  void slotCommandQuit();

  void slotViewToolBar();
  void slotViewStatusBar();
  void slotViewDirectoryLine();
  void slotViewCommandLine(bool toggle=true);

  void slotBookmarkAdd();
  void slotBookmarkRemove();
  void slotBookmarkChosen(int id);

  void slotOptionsGeneral();
  void slotOptionsFont();
  void slotOptionsKey();
  void slotOptionsColors();

  void slotRightHidden();
  void slotRightSortName();
  void slotRightSortSize();
  void slotRightSortAttrib();
  void slotRightSortOwner();
  void slotRightSortGroup();
  void slotRightSortLastModTime();
  void slotRightSortLastReadTime();
  void slotRightRefresh();


  void slotStatusMsg(const char *text);
  void slotStatusHelpMsg(const char *text);

  void slotChangeWindows();
  void slotDirectoryChanged(QString);

  void slotUp() {_ActListBox->slotUp();}
  void slotDown() {_ActListBox->slotDown();}
  void slotLeft() {_ActListBox->slotLeft();}
  void slotRight() {_ActListBox->slotRight();}
  void slotPageUp() {_ActListBox->slotPageUp();}
  void slotPageDown() {_ActListBox->slotPageDown();}
  void slotScrollBegin() {_ActListBox->slotScrollBegin();}
  void slotScrollEnd() {_ActListBox->slotScrollEnd();}
  void slotInvertSelection() {_ActListBox->slotInvertSelection();}
  void slotDirectoryUp() {_ActListBox->slotDirectoryUp();}
  void slotEnter() {_ActListBox->slotEnter();}
  void slotResetSubString() {_ActListBox->slotResetSubString();}

  void slotLeftBoxFocusIn();
  void slotRightBoxFocusIn();
  void slotCommandLineFocusIn();
  void slotCommandLineFocusOut();

  void slotExecuteFile(QString File);

  void slotSelectionChanged();
  void slotCursorChanged();

protected:
  void enableMessages(bool bEnable);
protected:
signals:
  void currentDirChanged(QString DirName);
private:
	/** the key accelerator container */
	KAccel* key_accel;

  //menu structure
  QPopupMenu *_left_menu;
	QPopupMenu *_right_menu;
	QPopupMenu *_options_menu;
	QPopupMenu *_view_menu;
  QPopupMenu *_command_menu;
  QPopupMenu *_help_menu;
  QPopupMenu *_bookmark_menu;
  /** doc represents your actual document and is created only once. It keeps
    * information such as filename and does the serialization of your files.
    */
  KNCDoc *doc;

	/** the configuration object of the application */
	KConfig* _config;
	
  MyListView *_LeftBox, *_RightBox, *_ActListBox;
  KNewPanner* _OuterPanner, *_Panner, *_CommandPanner;
  QMultiLineEdit* _ProtocolView;
  MyCommandWidget* _CommandWidget;
  bool  _Act;
  QString _Editor, _Viewer, _Find;
  bool _bShowDeleteMessageBox, _bExecKfm;
  bool _bViewDirectoryLine, _bViewStatusBar, _bViewToolbar, _bViewCommandLine;
  int _CommandPannerPos;

  int              _nBookmarks;
  QString          _bookmarks[MAX_BOOKMARKS];

};
 
#endif // KNC_H











