/***************************************************************************
                          knc_command.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Aug 19 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <iostream.h>
#include <sys/stat.h>

#include <qdir.h>
#include <qmessagebox.h>

#include <knc.h>
#include <kfm.h>

#include "mylistview.h"
#include "resource.h"
#include "progressdialog.h"
#include "deletedialog.h"
#include "selectdialog.h"
#include "attributedialog.h"
#include "usermenudialog.h"
#include "pluginadmin.h"
#include "helpfunc.h"
#include "mycommandline.h"

void KNCApp::select(bool Selection, QString Text)
{
 SelectDialog Select (this, i18n("Selection"), Text,"*");
 if (Select.exec())
 {
   QRegExp RegExp(Select.getText(), true, true);
   if (!_ActListBox->select(RegExp, Selection))
   {
     QMessageBox::information( this, i18n("knc"),
        i18n("The Wildcard you entered is not correct.") );
     return;
   }
 }
}

void KNCApp::slotCommandSelect()
{
 select(true, i18n("Select Files:"));
}

void KNCApp::slotCommandUnselect()
{
  select(false, i18n("Unselect Files:"));
}


void KNCApp::slotCommandView()
{
  MyListViewItem* Item = (MyListViewItem*) _ActListBox->currentItem();
  QString command = _Viewer + " \"" + Item->getFileInfo()->getFile() + "\" &";
  system(command);
}

void KNCApp::slotCommandEdit()
{
  MyListViewItem* Item = (MyListViewItem*) _ActListBox->currentItem();
  QString command = _Editor + " \"" + Item->getFileInfo()->getFile() + "\" &";
  system(command);
}

void KNCApp::createSourceTargetList(QList<ST>& SourceTargetList, QList<BaseFileInfo>& FileList, QString SourcePath, QString TargetPath, QString Pattern)
{
 QListIterator<BaseFileInfo> it(FileList);
 ST cur;
 QString SubDirPath, SubDir;
 QString NewName;
 while(it.current())
 {
   QString Target = it.current()->dirPath();
   Target.remove(0, SourcePath.length());
   if (Target == "") SubDir = "";

   int i = Target.find("/");
   if (i != -1) SubDirPath = Target.right(Target.length() - i - 1);
   else SubDirPath = "";

   Target = TargetPath + SubDir + SubDirPath;
   NewName = it.current()->fileName();
   if (Pattern != "*")
   {
     if (SubDir == "")
     {
       int pos;
       if ((pos = Pattern.find("*")) != -1)
       {
         NewName.prepend(Pattern.left(pos));
         NewName.append(Pattern.right(Pattern.length() - pos - 1));
       }
       else NewName = Pattern;
     }
   }
   if (it.current()->isDir())
   {
     cur.Info = it.current();

     QString Temp = it.current()->dirPath();// + "/";
     Temp.remove(0, SourcePath.length());

     if (Temp == "")
       SubDir = NewName + "/";

     cur.Source = it.current()->filePath();
     cur.Target = Target + NewName;
     SourceTargetList.append (new ST(cur));
   }
   else
   {
     cur.Info = it.current();
     cur.Source = it.current()->filePath();
     cur.Target = Target + NewName;
     SourceTargetList.append (new ST(cur));
   }
   ++it;
 }
}

const int SET = 0;
const int UNSET = 1;
const int NOSTATE = 2;
const int UNDEF = 3;

void setAttrib (char& RWE, int Attrib, int QFileInfoAttrib )
{
   if (RWE != NOSTATE)
   {
     if (Attrib & QFileInfoAttrib)
     {
       if (RWE != UNSET) RWE = SET;
       else RWE = NOSTATE;
     }
     else
     {
       if (RWE != SET) RWE = UNSET;
       else RWE = NOSTATE;
     }
   }
}

void KNCApp::slotCommandChangeAttributes()
{

 MyListViewItem* Item = (MyListViewItem*) _ActListBox->currentItem();
 double countfiles, sizeAll;
 QList<BaseFileInfo> FileList;
 char ReadUser = UNDEF, ReadGroup = UNDEF, ReadOther = UNDEF;
 char WriteUser = UNDEF, WriteGroup = UNDEF, WriteOther = UNDEF;
 char ExecuteUser = UNDEF, ExecuteGroup = UNDEF, ExecuteOther = UNDEF;


 BasePlugin* Plugin = PluginAdmin::instance().getResponsiblePlugin(_ActListBox->getDir(),"");
 int FO = BasePlugin::ChangeAttribute;
 if (!Plugin->sourceBeforeFileOperation(_ActListBox->getDir(), FO)) return;


 _ActListBox->createSelectedList(FileList, countfiles, sizeAll, false, true);
 QListIterator<BaseFileInfo> it(FileList);
 QString Owner, Group;

 while(it.current())
 {
   int Attrib = it.current()->getAttribute();

   setAttrib(ReadUser, Attrib, QFileInfo::ReadUser);
   setAttrib(WriteUser, Attrib, QFileInfo::WriteUser);
   setAttrib(ExecuteUser, Attrib, QFileInfo::ExeUser);

   setAttrib(ReadGroup, Attrib, QFileInfo::ReadGroup);
   setAttrib(WriteGroup, Attrib, QFileInfo::WriteGroup);
   setAttrib(ExecuteGroup, Attrib, QFileInfo::ExeGroup);

   setAttrib(ReadOther, Attrib, QFileInfo::ReadOther);
   setAttrib(WriteOther, Attrib, QFileInfo::WriteOther);
   setAttrib(ExecuteOther, Attrib, QFileInfo::ExeOther);

   if (Owner == NULL) Owner = it.current()->getOwner();
   if (Owner != it.current()->getOwner()) Owner = "";

   if (Group == NULL) Group = it.current()->getGroup();
   if (Group != it.current()->getGroup()) Group = "";

   ++it;
 }

 AttributeDialog ADialog(this, i18n("Change Attributes"));
 ADialog.setReadUser(ReadUser);
 ADialog.setWriteUser(WriteUser);
 ADialog.setExecuteUser(ExecuteUser);

 ADialog.setReadGroup(ReadGroup);
 ADialog.setWriteGroup(WriteGroup);
 ADialog.setExecuteGroup(ExecuteGroup);

 ADialog.setReadOther(ReadOther);
 ADialog.setWriteOther(WriteOther);
 ADialog.setExecuteOther(ExecuteOther);

 QStrList OwnerList, GroupList;
 bool OwnerEnabled = Plugin->getOwners(OwnerList);
 bool GroupEnabled = Plugin->getGroups(GroupList);

 ADialog.setOwner(Owner, OwnerList, OwnerEnabled);
 ADialog.setGroup(Group, GroupList, GroupEnabled);

 if (!ADialog.exec()) return;
 it.toFirst();
 int SetAttrib = 0;
 int UnsetAttrib = 0;
 if (ADialog.getReadUser() == SET)  SetAttrib |= QFileInfo::ReadUser;
 if (ADialog.getReadUser() == UNSET) UnsetAttrib |= QFileInfo::ReadUser;
 if (ADialog.getWriteUser() == SET)  SetAttrib |= QFileInfo::WriteUser;
 if (ADialog.getWriteUser() == UNSET) UnsetAttrib |= QFileInfo::WriteUser;
 if (ADialog.getExecuteUser() == SET) SetAttrib |= QFileInfo::ExeUser;
 if (ADialog.getExecuteUser() == UNSET)UnsetAttrib |= QFileInfo::ExeUser;

 if (ADialog.getReadGroup() == SET) SetAttrib |= QFileInfo::ReadGroup;
 if (ADialog.getReadGroup() == UNSET) UnsetAttrib |= QFileInfo::ReadGroup;
 if (ADialog.getWriteGroup() == SET) SetAttrib |= QFileInfo::WriteGroup;
 if (ADialog.getWriteGroup() == UNSET) UnsetAttrib |= QFileInfo::WriteGroup;
 if (ADialog.getExecuteGroup() == SET) SetAttrib |= QFileInfo::ExeGroup;
 if (ADialog.getExecuteGroup() == UNSET) UnsetAttrib |= QFileInfo::ExeGroup;

 if (ADialog.getReadOther() == SET)  SetAttrib |= QFileInfo::ReadOther;
 if (ADialog.getReadOther() == UNSET) UnsetAttrib |= QFileInfo::ReadOther;
 if (ADialog.getWriteOther() == SET) SetAttrib |= QFileInfo::WriteOther;
 if (ADialog.getWriteOther() == UNSET) UnsetAttrib |= QFileInfo::WriteOther;
 if (ADialog.getExecuteOther() == SET) SetAttrib |= QFileInfo::ExeOther;
 if (ADialog.getExecuteOther() == UNSET) UnsetAttrib |= QFileInfo::ExeOther;

 while(it.current())
 {
   int Attrib = it.current()->getAttribute();
   Attrib |= SetAttrib;
   Attrib &= ~UnsetAttrib;
   it.current()->setAttribute(Attrib);

   if (ADialog.getOwner() != Owner) it.current()->setOwner(ADialog.getOwner());
   if (ADialog.getGroup() != Group) it.current()->setGroup(ADialog.getGroup());

   ++it;
 }

 refresh(_Act, Item->getFileInfo()->fileName());
 Plugin->sourceAfterFileOperation(_ActListBox->getDir(), FO);

}

void KNCApp::slotCommandCopy()
{
 copyMove(false);
}

void KNCApp::slotCommandMove()
{
 copyMove(true);
}

//this method copies the selected Files. If bmove is set, the source is also deleted
void KNCApp::copyMove(bool bMove)
{
 double countfiles = 0;
 double sizeAll = 0;
 QString Message;
 MyListView *SBox, *TBox;
 bool   bOverwrite = false;
 bool   bStop = false;
 bool   bRenamed = false;
 bool OpSuccess;

 QString SourcePath, TargetPath, Pattern;

 if (_Act)
 {
   SBox = _LeftBox; TBox = _RightBox;
   SourcePath = _LeftBox->getDir();
   TargetPath = _RightBox->getDir() += "*";
 }
 else
 {
   SBox = _RightBox; TBox = _LeftBox;
   SourcePath = _RightBox->getDir();
   TargetPath = _LeftBox->getDir() += "*";
 }

 QString Title;
 if (bMove) Title = i18n("Move"); else Title = i18n("Copy");

 SelectDialog SDialog(this, Title, Title + i18n(" files to:"), TargetPath);
 if (!SDialog.exec()) return;

 TargetPath = SDialog.getText();
 int Cut = TargetPath.findRev("/");
 if (Cut != -1)
 {
   Pattern = TargetPath.right(TargetPath.length() - Cut - 1);
   TargetPath = TargetPath.left(Cut + 1);
 }
 else
 {
   Pattern = TargetPath;
   TargetPath = SourcePath;
 }

 //check if the plugins permit the operation
 int FO = 0;
 if (bMove)                    FO |= BasePlugin::FileDelete;
 else                          FO |= BasePlugin::FileCopy;
 if (SourcePath == TargetPath) FO |= BasePlugin::FileRename;

 BasePlugin* SourcePlugin = PluginAdmin::instance().getResponsiblePluginInstance(SourcePath,"");
 BasePlugin* TargetPlugin = PluginAdmin::instance().getResponsiblePluginInstance(TargetPath,"");
 if ((SourcePlugin == NULL) || (TargetPlugin == NULL))
 {
   QMessageBox::information( this, i18n("KNC"), i18n("Sorry, your operation is not supported."),
                                       QMessageBox::Ok|QMessageBox::Default);
   return;
 }
 if (!SourcePlugin->sourceBeforeFileOperation(SourcePath, FO)) return;
 if (!TargetPlugin->targetBeforeFileOperation(TargetPath, FO)) return;

 QList<BaseFileInfo> FileList;
 FileList.setAutoDelete(true);

 QList<BaseFileInfo> Dirs;


 if (SourcePath == TargetPath)
   _ActListBox->createSelectedList(FileList, countfiles, sizeAll, false, true);
 else
   _ActListBox->createSelectedList(FileList, countfiles, sizeAll, true, true);

 QList<ST> SourceTargetList;
 SourceTargetList.setAutoDelete(true);

 createSourceTargetList(SourceTargetList, FileList, SourcePath, TargetPath, Pattern);

 //unselect all items in  ActListBox
 _ActListBox->select(QRegExp("*", true, true), false);

 //show the progress dialog
 ProgressDialog PDialog(this);
 PDialog.center(this);
 PDialog.show();
 PDialog.setAction(Title + i18n(" File:"));
 PDialog.set(countfiles, sizeAll);
 QListIterator<ST> it(SourceTargetList);
 qApp->processEvents(1000);

 //let's go
 while(it.current())
 {
   QString Source = it.current()->Source;
   QString Target = it.current()->Target;
   QString SPath, SName;
   cutUpPath(Source, SPath, SName);
   QString TPath, TName;
   cutUpPath(Target, TPath, TName);

   PDialog.newFile(Source, Target, it.current()->Info->size());
   //this call is needed for the correct drawing of the dialog
   qApp->processEvents(1000);

   if ((SourcePath == TargetPath) && (bMove)) //rename is faster than move
   {
     OpSuccess = it.current()->Info->rename(&PDialog, Target, bOverwrite, bStop);
     if (OpSuccess) removeItem(SPath, SName);
     bRenamed = true;
   }
   else
   {
     if (TargetPlugin->getType() == "GeneralFilePlugin")
     {
       OpSuccess = it.current()->Info->copy(&PDialog, Target, bOverwrite, bStop);

       if (OpSuccess)
       {
         int Attrib = it.current()->Info->getAttribute();
         chmod(Target, Attrib);
       }
     }
     else if (SourcePlugin->getType() == "GeneralFilePlugin")
      OpSuccess = TargetPlugin->copyIn(&PDialog, *it.current(), bOverwrite, bStop);
   }

   if (OpSuccess)
    insertItem(TPath, TName);

   if ((bMove) && (!bRenamed) && (OpSuccess))
   {
    //Put the old directory in the list for deleting
     if (it.current()->Info->isDir())
       Dirs.append(it.current()->Info);
     else
     {
       if (it.current()->Info->remove(NULL, bStop))
       {
         QString SPath, SName;
         cutUpPath(Source, SPath, SName);
         removeItem(SPath, SName);
       }
     }
   }
   if (bStop) goto end;
   ++it;
 }

 //at last we have to delete the directories in reversed order, but only if they are not renamed!
 if ((bMove) && (!bRenamed))
 {
   QListIterator<BaseFileInfo> it (Dirs);
   it.toLast();
   while (it.current())
   {
     if (it.current()->remove(NULL, bStop))
     {
       if (bStop) goto end;
       removeItem(it.current()->dirPath(), it.current()->fileName());
     }
     --it;
   }
 }

 SourcePlugin->sourceAfterFileOperation(SourcePath, FO);
 TargetPlugin->targetAfterFileOperation(TargetPath, FO);
 end:
 return;
}

void KNCApp::refresh(bool Left, QString CurrentItemName)
{
 if (Left) _LeftBox->refresh(CurrentItemName);
 else      _RightBox->refresh(CurrentItemName);
}

void KNCApp::slotCommandNewDir()
{
 BasePlugin* Plugin = PluginAdmin::instance().getResponsiblePlugin(_ActListBox->getDir(),"");
 int FO = BasePlugin::NewDirectory;
 if (!Plugin->sourceBeforeFileOperation(_ActListBox->getDir(), FO)) return;


 SelectDialog Dialog(this, i18n("New Directory"), i18n("New Directory Name:"));

 if (Dialog.exec())
 {
   if (!Plugin->makeDirectory(_ActListBox->getDir(), Dialog.getText())) return;
   refresh(_Act, Dialog.getText());
 }
 Plugin->sourceAfterFileOperation(_ActListBox->getDir(),FO);
}

void KNCApp::removeItem(QString Dir, QString Name)
{
  BasePlugin* Plugin = PluginAdmin::instance().getResponsiblePlugin(Dir,"");
  Plugin->removeEntry(_LeftBox, Dir, Name);
  Plugin->removeEntry(_RightBox, Dir, Name);
}

void KNCApp::insertItem(QString Dir, QString Name)
{
  BasePlugin* Plugin = PluginAdmin::instance().getResponsiblePlugin(Dir,"");
  Plugin->insertEntry(_LeftBox, Dir, Name);
  Plugin->insertEntry(_RightBox, Dir, Name);
}



void KNCApp::slotCommandDelete()
{
 QList<BaseFileInfo> FileList;
 FileList.setAutoDelete(true);
 double countfiles = 0;
 double sizeAll = 0;
 bool bStop = false;

 BasePlugin* Plugin = PluginAdmin::instance().getResponsiblePlugin(_ActListBox->getDir(),"");
 int FO = BasePlugin::FileDelete;
 if (!Plugin->sourceBeforeFileOperation(_ActListBox->getDir(), FO)) return;


 _ActListBox->createSelectedList(FileList,countfiles, sizeAll, true, true);
 QString Message;
 if (_bShowDeleteMessageBox)
 {
   QString D1;
   int clicked = QMessageBox::warning( this, i18n("KNC"), i18n("Really delete ") + D1.setNum(countfiles,'g',25) + i18n(" files with ") + D1.setNum(sizeAll,'g',25) + i18n(" bytes space?"),
                                       QMessageBox::Ok|QMessageBox::Default, QMessageBox::Cancel);
   if (clicked == QMessageBox::Cancel) return;
 }

 DeleteDialog Dialog(this, i18n("Delete"));
 Dialog.center(this);
 Dialog.show();
 Dialog.set(countfiles, sizeAll);
 qApp->processEvents();

 QListIterator<BaseFileInfo> it(FileList);

 it.toLast();

 while(it.current())
 {
   QString T1 = it.current()->filePath();
   Dialog.newFile(T1);

   //this call is needed for the correct drawing of the dialog
   qApp->processEvents();

   if (it.current()->remove(&Dialog, bStop))
     removeItem(it.current()->dirPath(), it.current()->fileName());

   if (bStop) goto end;
   --it;
 }
 Plugin->sourceAfterFileOperation(_ActListBox->getDir(), FO);
 end:
 return;
// refresh(_Act);
}

void KNCApp::slotCommandOpenTerminal()
{
  QString command;
  command = "cd \"" + _ActListBox->getDir() + "\";kvt&";
  system(command);
}

void KNCApp::slotCommandOpenFindUtility()
{
  QString command;
  command = "cd \"" + _ActListBox->getDir() + "\";kfind&";
  system(command);
}


void KNCApp::slotCommandQuit()
{
	saveOptions();
  qApp->quit();
}


void KNCApp::exec(QString command)
{
  if (_bExecKfm)
  {
    KFM kfm;
	  kfm.exec(command.data(),0L);
  }
  else slotCommandExecuteWith();
}


//is executed when the user doubleclicks on a datafile
void KNCApp::slotExecuteFile(QString File)
{
  exec(File);
}

void KNCApp::slotCommandExecuteWith()
{
 MyListViewItem* Item = (MyListViewItem*) _ActListBox->currentItem();
 if (Item)
 {
   if (Item->getFileInfo()->isFile())
   {
//    QString Message;
    SelectDialog Select (this, i18n("Open with"), i18n("Open with"), "\"" +  Item->getFileInfo()->fileName() + "\"" );
    if (!Select.exec()) return;
    system (Select.getText() + "&");
//      QMessageBox::information( this, i18n("KNC"), Message.sprintf("Unable to execute statement:\n%s ", Select.getText().data()),
//                              QMessageBox::Ok);

   }
 }
}


void KNCApp::slotCommandOpenUserMenu()
{
  UserMenuDialog Dialog (this, 0, _config, _ActListBox);
  if (!Dialog.exec()) return;
}

void KNCApp::slotCommandInvokeCommandLine()
{
  if (!_bViewCommandLine) slotViewCommandLine();
  if (_CommandWidget->getCommandLine()->hasFocus())
   _LeftBox->setFocus();
  else
    _CommandWidget->getCommandLine()->setFocus();
}











