/***************************************************************************
                          mylistview.h  -  description                              
                             -------------------                                         
    begin                : Mon Aug 9 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef MYLISTVIEW_H
#define MYLISTVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qlist.h>
#include <qlistview.h>
#include <qdir.h>
#include <qregexp.h>

//#include "knc.h"
//#include "generalfileinfo.h"
#include "baseplugin.h"


class QMultiLineEdit;
class MyListViewItem;
class BaseFileInfo;
class KTar;
/**
  *@author Henrik Stormer
  */

typedef QListIterator<BasePlugin> PluginIterator;

class MyListView : public QListView
{
 Q_OBJECT

 friend MyListViewItem;

 public:
 MyListView ( QWidget * parent, const char * name, QMultiLineEdit* ProtocolView );
 ~MyListView();

  void traverseDirectoryDown (BaseFileInfo* Info);
  void traverseDirectoryUp   ();
  void setDirectory(QString absPath) {_Dir = absPath; traverseDirectory(_Dir);}

  bool select(QRegExp RegExp, bool Selection);

  void refresh(QString CurrentItemName="");

  void createSelectedList(QList<BaseFileInfo>& FileList, double& countfiles, double& sizeAll, bool bIncludeSubdirs, bool createList);

  QString getDir() {return _Dir;}
  int getNameColumn() {return _ncName;}
  int getSizeColumn() {return _ncSize;}
  int getAttribColumn() {return _ncAttrib;}
  int getLastReadColumn() {return _nclRead;}
  int getLastModifiedColumn() {return _nclMod;}

  void setShowHiddenFiles (bool Show) {_bShowHiddenFiles = Show; refresh();}
  bool getShowHiddenFiles () {return _bShowHiddenFiles;}

  void setSelectionIncludeSubdirs(bool Sel) {_bSelectionIncludeSubdirs = Sel;}
  bool getSelectionIncludeSubdirs() {return _bSelectionIncludeSubdirs;}

  //some methods for selection
  virtual void setSelected( QListViewItem * item, bool selected );
  bool isSelection() {return !_SelectedList.isEmpty();}
  double getSelectionFileCount() ;
  double getSelectionFileSize();

  MyListViewItem* insertItem(BaseFileInfo* f);
  void            removeItem(MyListViewItem* Item);
  MyListViewItem* findItem  (BaseFileInfo* f);
  MyListViewItem* findItem  (QString Name);

  void   setFocusColor(QColor& FocusColor) {_FocusColor = FocusColor;}
  void   setFocusCursorColor(QColor& FocusCursorColor) {_FocusCursorColor = FocusCursorColor;}

  QColor& getFocusColor() {return _FocusColor;}
  QColor& getFocusCursorColor() {return _FocusCursorColor;}

 protected:
  signals:
   void focusIn();
   void focusOut();
   void directoryChanged (QString);
   void executeFile (QString);
   void cursorChanged();
   void selectChanged();

 protected:

   //these functiones are used for traversing the directory including subdirectories
   void traverseDirectory(QString Dir, QString CurrentItemName="");

   void makeItemVisible(MyListViewItem* Item);
   bool matchItem(MyListViewItem* Item, QString SubString);

   MyListViewItem* findSubStringItem(bool ascending, QString SubString);

   virtual void focusInEvent( QFocusEvent * e ) {emit focusIn();QListView::focusInEvent(e);}
   virtual void focusOutEvent( QFocusEvent * e ) {emit focusOut();QListView::focusOutEvent(e);}
   virtual void keyPressEvent ( QKeyEvent * e);
   virtual void mousePressEvent (QMouseEvent* e);
   virtual void mouseReleaseEvent (QMouseEvent* e);
public slots:
   void slotEnter();
   void slotUp();
   void slotDown();
   void slotLeft();
   void slotRight();
   void slotPageUp();
   void slotPageDown();
   void slotScrollBegin();
   void slotScrollEnd();
   void slotInvertSelection();
   void slotDirectoryUp();
   void slotResetSubString();
  private:
   QString                _Dir;
   int                    _ncName, _ncSize, _ncAttrib, _nclMod, _nclRead, _ncOwner, _ncGroup;
   bool                   _bShowHiddenFiles;
   QString                _SubString;
   QList<MyListViewItem>  _SelectedList;
   bool                   _bSelectionDirty;
   double                 _SelectionFileCount;
   double                 _SelectionFileSize;
   QMultiLineEdit*        _ProtocolView;
   bool                   _bSelectionIncludeSubdirs;
   struct SC
   {
     double size;
     double count;
   };
   QDict<struct SC>       _RecurseSubdir;
   BasePlugin*            _ActivePlugin;
   QColor                 _FocusColor, _FocusCursorColor;
};

#endif




