/***************************************************************************
                          tarfileinfo.cpp  -  description                              
                             -------------------                                         
    begin                : Tue Oct 12 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "tarfileinfo.h"
#include "qfileinfo.h"
#include "qmessagebox.h"
#include "qdir.h"
#include "kapp.h"
#include "qstring.h"

TarEntryInfo::TarEntryInfo(QString Path, KTarEntry* Entry) : BaseFileInfo(), _Entry(Entry)
{
  _Path = Path;
  if (_Entry)
  {
    _Name = _Entry->name();
    _Mode = _Entry->permissions();
    _Owner = Entry->user();
    _Group = Entry->group();
    _LastRead = _Entry->datetime();
    _LastModified = _Entry->datetime();
  }
}

QString TarEntryInfo::getAttrib() const
{
 QString Attrib;
  if (QFileInfo::ReadUser & _Mode) Attrib += "r"; else Attrib += "-";
  if (QFileInfo::WriteUser & _Mode) Attrib += "w"; else Attrib += "-";
  if (QFileInfo::ExeUser & _Mode) Attrib += "x"; else Attrib += "-";

  if (QFileInfo::ReadGroup & _Mode) Attrib += "r"; else Attrib += "-";
  if (QFileInfo::WriteGroup & _Mode) Attrib += "w"; else Attrib += "-";
  if (QFileInfo::ExeGroup & _Mode) Attrib += "x"; else Attrib += "-";

  if (QFileInfo::ReadOther & _Mode) Attrib += "r"; else Attrib += "-";
  if (QFileInfo::WriteOther & _Mode) Attrib += "w"; else Attrib += "-";
  if (QFileInfo::ExeOther & _Mode) Attrib += "x"; else Attrib += "-";

  return Attrib;
}


TarEntryInfo::~TarEntryInfo()
{
}

TarFileInfo::TarFileInfo(QString Path, KTarFile* File)
   : TarEntryInfo(Path, File), _TarFile(File)
{
  _dSize = File->size();
}

TarFileInfo::~TarFileInfo()
{}

TarDirInfo::TarDirInfo(QString Path, KTarDirectory* Dir)
   : TarEntryInfo(Path, Dir), _TarDirectory(Dir), _bSizeSet(false)
{
}

TarDirInfo::TarDirInfo(QString Path, QString Name, long Mode, QDateTime LastRead, QDateTime LastModified)
  : TarEntryInfo(Path, NULL)
{
  _Name = Name;
  _Mode = Mode;
  _LastRead = LastRead;
  _LastModified = LastModified;
}

TarDirInfo::~TarDirInfo()
{}


QString TarEntryInfo::infoAsString ()
{
 return "TAR: " + _Name + " " + getSize() + " " + _Attrib;
}

void TarEntryInfo::setAttribute(long Attribute)
{
}
long TarEntryInfo::getAttribute() const
{
 return _Mode;
}

bool TarFileInfo::copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop)
{
  QString Source = filePath();
  up:
  QFile FTarget (Target);
  QString Message;
  if (Source == Target)
  {
     int clicked = QMessageBox::critical( NULL, i18n("KNC"), Message.sprintf(i18n("You cannot copy a file to itself.")),
                                          QMessageBox::Ok, QMessageBox::Abort);
     if (clicked == QMessageBox::Ok)  return false;
     if (clicked == QMessageBox::Abort) {stop = true; goto end;}
  }

  if ((FTarget.exists()) && (!bOverwrite))
  {

    QMessageBox mb( "KNC", Message.sprintf(i18n("File %s exists. Overwrite?"), Target.data()),QMessageBox::Critical,
                     QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Retry );
     mb.setButtonText( QMessageBox::Retry, "All" );
     switch( mb.exec() ) {case QMessageBox::No: return false; break;
       case QMessageBox::Retry: bOverwrite = true; break;}
  }
  if (!FTarget.open(IO_WriteOnly))
  {
    int clicked = QMessageBox::critical( NULL, i18n("KNC"), Message.sprintf(i18n("Unable to open File %s."), Target.data()),
                                         QMessageBox::Abort, QMessageBox::Retry);
    if (clicked == QMessageBox::Abort) {stop = true; goto end;}
    if (clicked == QMessageBox::Retry) goto up;
  }

    FTarget.writeBlock(_TarFile->data().data(), _TarFile->data().size());
    if (PDialog) PDialog->addSize(_TarFile->data().size());
  end:
  FTarget.close();
  return true;
}

bool TarEntryInfo::rename(ProgressDialog* PFile, QString Target, bool& bOverwrite, bool& stop)
{
 return false;
}
bool TarEntryInfo::remove(DeleteDialog* PFile, bool& stop)
{
 return false;
}

bool TarEntryInfo::copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop)
{
 return false;
}

bool TarDirInfo::copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop)
{
  QString Message;
 up:
  QDir Dir(Target);
  QString Name = Dir.dirName();
  Dir.cdUp();
  if (!Dir.exists()) goto error;
  if (!Dir.mkdir(fileName())) goto error;
  return true;
  error:
   int clicked = QMessageBox::critical( NULL, i18n("KNC"), Message.sprintf(i18n("Unable to create Directory %s."), filePath().data()),
                                       QMessageBox::Ok, QMessageBox::Retry, QMessageBox::Abort);

   if (clicked == QMessageBox::Retry) goto up;
   if (clicked == QMessageBox::Abort) {stop = true;return false;}
}


QByteArray TarFileInfo::data()
{
 return _TarFile->data();
}

QString TarFileInfo::getFile()
{
  QString TempFileName = tmpnam (NULL);
  QFile TempFile(TempFileName);
  TempFile.open(IO_WriteOnly);
  TempFile.writeBlock(_TarFile->data().data(), _TarFile->data().size());
  TempFile.close();
  return TempFileName;
}


