/***************************************************************************
                          tarfileinfo.h  -  description                              
                             -------------------                                         
    begin                : Tue Oct 12 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef TARFILEINFO_H
#define TARFILEINFO_H

#include <basefileinfo.h>
#include <ktar.h>
/**                        , _
  *@author Henrik Stormer
  */

class TarEntryInfo : public BaseFileInfo
{
public:
  TarEntryInfo(QString Path, KTarEntry* Entry);
	virtual ~TarEntryInfo();

  virtual QString infoAsString ();
  virtual QString type() {return "TarFile";}

  virtual QString getAttrib() const;
  virtual void    setAttribute(long Attribute);
  virtual long    getAttribute() const;

  virtual bool accessRead() const {return true;}
  virtual bool accessWrite() const  {return true;}
  virtual bool accessExecute() const {return true;}

  virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
  virtual bool rename(ProgressDialog* PFile, QString Target, bool& bOverwrite, bool& stop);
  virtual bool remove(DeleteDialog* PFile, bool& stop);

  protected:
   virtual KTarEntry* getEntry() {return _Entry;}
   long        _Mode;
  private:
    KTarEntry* _Entry;
};

class TarFileInfo : public TarEntryInfo
{
 public:
//	TarFileInfo() : TarEntryInfo(NULL) {}
  TarFileInfo(QString Path, KTarFile* File);
	virtual ~TarFileInfo();

  virtual QString type() {return "TarFile";}
  virtual TarFileInfo* clone ()  {return new TarFileInfo(*this);}

  virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
  virtual QByteArray data();

  virtual KTarFile* getTarEntry() {return _TarFile;}
  virtual QString getFile();
 private:
   KTarFile* _TarFile;
};


class TarDirInfo : public TarEntryInfo
{
 public:
//	TarDirInfo() : TarEntryInfo(), _bSizeSet(false) {}
  TarDirInfo(QString Path, KTarDirectory* Dir);
  TarDirInfo(QString Path, QString Name, long Mode, QDateTime LastAccess, QDateTime LastModified);
	virtual ~TarDirInfo();

  virtual QString getSize() const {return "<DIR>";}

  virtual QString type() {return "TarDir";}
  virtual TarDirInfo* clone ()  {return new TarDirInfo(*this);}

  virtual bool isFile() const {return false;}
  virtual bool isDir() const {return true;}

  QPixmap getIcon() {return kapp->getIconLoader()->loadMiniIcon("folder.xpm");}
  virtual bool copy(ProgressDialog* PDialog, QString Target, bool& bOverwrite, bool& stop);
  virtual QByteArray data() {return QByteArray();}

 protected:
  virtual KTarDirectory* getTarEntry() {return _TarDirectory;}
 private:
  KTarDirectory* _TarDirectory;
  bool _bSizeSet;
};

#endif