/***************************************************************************
                          tarfileplugin.cpp  -  description                              
                             -------------------                                         
    begin                : Tue Oct 12 1999                                           
    copyright            : (C) 1999 by Henrik Stormer                         
    email                : stormer@ifi.unizh.ch                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "tarfileplugin.h"
#include "ktar.h"
#include "qfile.h"
#include "knc.h"
#include "qmessagebox.h"
#include "helpfunc.h"

#define PRIORITY 5

TarFilePlugin::TarFilePlugin(TarFilePluginFactory* Factory) : BasePlugin(),
  _Factory(Factory), _TarFile(0)
{
}
TarFilePlugin::~TarFilePlugin()
{
 delete (_TarFile);
}

TarFilePluginFactory::TarFilePluginFactory() : BasePluginFactory(PRIORITY)
{}

TarFilePluginFactory::~TarFilePluginFactory()
{}


bool TarFilePluginFactory::isResponsible(QString Path, QString SubPath)
{
 if (Path.find("#tar:") != -1) return true;

 if (SubPath != "")
   if ((SubPath.find(".tar") != -1) || (SubPath.find(".tar.gz") != -1) || (SubPath.find(".tgz") != -1)) return true;

 return false;
}

BasePluginFactory* TarFilePlugin::getPluginFactory()
{
 return _Factory;
}

bool TarFilePlugin::isResponsible(QString Path, QString SubPath)
{
 QString FPath, Dummy;
 if (!ripPath(Path, "#tar:", FPath, Dummy))
   FPath = Path+SubPath;

 if (FPath == _FileName) return true;
 return false;
}

void TarFilePlugin::createTarFile(QString Path)
{
  QString FilePath, Dummy;
  if (!ripPath(Path, "#tar:", _FileName, Dummy)) return;
  _TarFile = new KTar(_FileName);
}

KTarDirectory* TarFilePlugin::gotoDir(QString SubDir)
{
  KTarDirectory* Dir = (KTarDirectory*)_TarFile->directory();
  int pos = 1;
  pos = SubDir.find("/",1);
  while (pos != -1)
  {
    QString NextPath = SubDir.left(pos);
    SubDir = SubDir.right(SubDir.length() - (pos + 1));
    Dir = (KTarDirectory*) Dir->entry(NextPath);
    pos = SubDir.find("/",1);
  }
  return Dir;
}

bool TarFilePlugin::create(QString Path, MyListView* ListBox, QString CurrentItemName, bool ShowHiddenFiles)
{
  if (!_TarFile) createTarFile(Path);
  if (!_TarFile->isOpened()) _TarFile->open(IO_ReadOnly);

  QString Dummy, SubPath;
  ripPath(Path, "#tar:", Dummy, SubPath);

  if ((_WorkingDir = gotoDir(SubPath)) == NULL) return false;

  ListBox->clear();

  QStringList Entries = _WorkingDir->entries();
  QListIterator<QString> it(Entries);
  BaseFileInfo* Entry;
  Entry = new TarDirInfo(Path, "..", 100, QDateTime::currentDateTime(), QDateTime::currentDateTime());
  MyListViewItem* Item = ListBox->insertItem(Entry);
  Item->setSelectable(false);
  while(it.current())
  {
    QString Name, Size, Attrib, LModified, LRead;
    if (_WorkingDir->entry(*it.current())->isDirectory())
       Entry = new TarDirInfo(Path, (KTarDirectory*)_WorkingDir->entry(*it.current()));
    if (_WorkingDir->entry(*it.current())->isFile())
       Entry = new TarFileInfo(Path, (KTarFile*)_WorkingDir->entry(*it.current()));

    MyListViewItem* Item = ListBox->insertItem(Entry);
    if (Entry->fileName() == CurrentItemName) ListBox->setCurrentItem(Item);
    ++it;
  }
  return true;
}

QString TarFilePlugin::cdUp(QString Path, QString& Name)
{
   QString SubPath, FilePath;
   ripPath(Path, "#tar:", FilePath, SubPath);
   if (SubPath == "/")
   {
     cutUpPath(FilePath, FilePath, Name);
     return FilePath;
   }
   else
   {
     cutUpPath(SubPath, SubPath, Name);
     return FilePath + "#tar:" + SubPath;
   }
}

QString TarFilePlugin::cdDown(QString Path, QString DownPath)
{
  int pos = Path.find("#tar:");
  if (pos != -1)
  {
     return Path + DownPath + "/";
  }
  else
  {
    return Path + DownPath + "#tar:/";
  }
}

void TarFilePlugin::recurseSubdir(QString Dir, QList<BaseFileInfo>& FileList, double& countFiles, double& sizeAll)
{
  TarEntryInfo* Entry;
  QString Dummy, SubPath;
  ripPath(Dir, "#tar:", Dummy, SubPath);

  KTarDirectory* TarDir = gotoDir(SubPath);

  QStringList Entries = TarDir->entries();
  QListIterator<QString> it(Entries);
  while(it.current())
  {
    if (TarDir->entry(*it.current())->isDirectory())
    {
       Entry = new TarDirInfo(Dir, (KTarDirectory*) TarDir->entry(*it.current()));
       FileList.append(Entry);
       recurseSubdir(Dir + Entry->fileName() + "/", FileList, countFiles, sizeAll);
    }
    if (TarDir->entry(*it.current())->isFile())
    {
       Entry = new TarFileInfo(Dir, (KTarFile*) TarDir->entry(*it.current()));
       FileList.append(Entry);
       countFiles++;
       sizeAll += Entry->size();
    }
    ++it;
  }
}

void TarFilePlugin::insertEntry(MyListView* ListBox, QString Dir, QString Name)
{
}
void TarFilePlugin::removeEntry(MyListView* ListBox, QString Dir, QString Name)
{
}


bool TarFilePlugin::copyIn(ProgressDialog* PDialog, ST st, bool& bOverwrite, bool& bStop)
{
}

bool TarFilePlugin::sourceBeforeFileOperation(QString Path, int Operation)
{
  QString Message;
 if (Operation & FileDelete) { Message = "You cannot delete files inside a tar file";}
 if (Operation & FileRename) { Message = "You cannot rename files inside a tar file";}
 if (Operation & ChangeAttribute) { Message = "You cannot change attributes inside a tar file";}
 if (Operation & NewDirectory) { Message = "You cannot create new directories inside a tar file";}
 if (Operation & FileCopy) {if (Message == NULL) return true;}

  QMessageBox::information( NULL, i18n("KNC"), Message, QMessageBox::Ok|QMessageBox::Default);
  return false;
}

bool TarFilePlugin::targetBeforeFileOperation(QString Path, int Operation)
{
  QString Message;
 if (Operation & FileCopy) { Message = "You cannot copy files to a tar file";}
 if (Operation & FileDelete) { Message = "You cannot delete files inside a tar file";}
 if (Operation & FileRename) { Message = "You cannot rename files inside a tar file";}
 if (Operation & ChangeAttribute) { Message = "You cannot change attributes inside a tar file";}
 if (Operation & NewDirectory) { Message = "You cannot create new directories inside a tar file";}

  QMessageBox::information( NULL, i18n("KNC"), Message, QMessageBox::Ok|QMessageBox::Default);
  return false;
}






























